@@ -350,7 +350,6 @@ create_screen_cast_dialog (ScreenCastSession     *session,
 {
   ScreenCastDialogHandle *dialog_handle;
   ExternalWindow *external_parent;
-  GdkDisplay *display;
   GdkSurface *surface;
   GtkWidget *fake_parent;
   GtkWindow *dialog;
@@ -367,13 +366,7 @@ create_screen_cast_dialog (ScreenCastSession     *session,
       external_parent = NULL;
     }
 
-  if (external_parent)
-    display = external_window_get_display (external_parent);
-  else
-    display = gdk_display_get_default ();
-  fake_parent = g_object_new (GTK_TYPE_WINDOW,
-                              "display", display,
-                              NULL);
+  fake_parent = g_object_new (GTK_TYPE_WINDOW, NULL);
   g_object_ref_sink (fake_parent);
 
   dialog = GTK_WINDOW (screen_cast_dialog_new (request->app_id,
@@ -399,7 +392,7 @@ create_screen_cast_dialog (ScreenCastSession     *session,
   if (external_parent)
     external_window_set_parent_of (external_parent, surface);
 
-  gtk_widget_show (GTK_WIDGET (dialog));
+  gtk_window_present (dialog);
 
   return dialog_handle;
 }
@@ -478,6 +471,24 @@ find_best_window_by_app_id_and_title (const char *app_id,
   return best_match;
 }
 
+void
+screen_cast_stream_info_free (ScreenCastStreamInfo *info)
+{
+  switch (info->type)
+    {
+    case SCREEN_CAST_SOURCE_TYPE_MONITOR:
+      g_clear_pointer (&info->data.monitor, monitor_free);
+      break;
+    case SCREEN_CAST_SOURCE_TYPE_WINDOW:
+      g_clear_pointer (&info->data.window, window_free);
+      break;
+    case SCREEN_CAST_SOURCE_TYPE_VIRTUAL:
+      break;
+    }
+
+  g_free (info);
+}
+
 static gboolean
 restore_stream_from_data (ScreenCastSession *screen_cast_session)
 
@@ -495,7 +506,8 @@ restore_stream_from_data (ScreenCastSession *screen_cast_session)
   if (!screen_cast_session->restored.data)
     return FALSE;
 
-  streams = g_ptr_array_new_with_free_func (g_free);
+  streams =
+    g_ptr_array_new_with_free_func ((GDestroyNotify) screen_cast_stream_info_free);
 
   g_variant_get (screen_cast_session->restored.data,
                  RESTORE_VARIANT_TYPE,
@@ -521,7 +533,7 @@ restore_stream_from_data (ScreenCastSession *screen_cast_session)
 
             info = g_new0 (ScreenCastStreamInfo, 1);
             info->type = SCREEN_CAST_SOURCE_TYPE_MONITOR;
-            info->data.monitor = monitor;
+            info->data.monitor = monitor_dup (monitor);
             info->id = id;
             g_ptr_array_add (streams, info);
           }
@@ -546,7 +558,7 @@ restore_stream_from_data (ScreenCastSession *screen_cast_session)
 
             info = g_new0 (ScreenCastStreamInfo, 1);
             info->type = SCREEN_CAST_SOURCE_TYPE_WINDOW;
-            info->data.window = window;
+            info->data.window = window_dup (window);
             info->id = id;
             g_ptr_array_add (streams, info);
           }
@@ -679,7 +691,8 @@ handle_select_sources (XdpImplScreenCast     *object,
     types = SCREEN_CAST_SOURCE_TYPE_MONITOR;
 
   if (!(types & (SCREEN_CAST_SOURCE_TYPE_MONITOR |
-                 SCREEN_CAST_SOURCE_TYPE_WINDOW)))
+                 SCREEN_CAST_SOURCE_TYPE_WINDOW |
+                 SCREEN_CAST_SOURCE_TYPE_VIRTUAL)))
     {
       g_warning ("Unknown screen cast source type");
       response = 2;
@@ -792,9 +805,6 @@ out:
 static void
 on_gnome_screen_cast_enabled (GnomeScreenCast *gnome_screen_cast)
 {
-  int gnome_api_version;
-  ScreenCastSourceType available_source_types;
-  ScreenCastCursorMode available_cursor_modes;
   g_autoptr(GError) error = NULL;
 
   impl = G_DBUS_INTERFACE_SKELETON (xdp_impl_screen_cast_skeleton_new ());
@@ -806,25 +816,13 @@ on_gnome_screen_cast_enabled (GnomeScreenCast *gnome_screen_cast)
   g_signal_connect (impl, "handle-start",
                     G_CALLBACK (handle_start), NULL);
 
-  gnome_api_version = gnome_screen_cast_get_api_version (gnome_screen_cast);
-
-  available_source_types = SCREEN_CAST_SOURCE_TYPE_MONITOR;
-  if (gnome_api_version >= 2)
-    available_source_types |= SCREEN_CAST_SOURCE_TYPE_WINDOW;
   g_object_set (G_OBJECT (impl),
-                "available-source-types", available_source_types,
-                NULL);
-
-  available_cursor_modes = SCREEN_CAST_CURSOR_MODE_NONE;
-  if (gnome_api_version >= 2)
-    {
-      available_cursor_modes |= SCREEN_CAST_CURSOR_MODE_HIDDEN |
-                                SCREEN_CAST_CURSOR_MODE_EMBEDDED |
-                                SCREEN_CAST_CURSOR_MODE_METADATA;
-    }
-
-  g_object_set (G_OBJECT (impl),
-                "available-cursor-modes", available_cursor_modes,
+                "available-source-types", SCREEN_CAST_SOURCE_TYPE_MONITOR |
+                                          SCREEN_CAST_SOURCE_TYPE_WINDOW |
+                                          SCREEN_CAST_SOURCE_TYPE_VIRTUAL,
+                "available-cursor-modes", SCREEN_CAST_CURSOR_MODE_HIDDEN |
+                                          SCREEN_CAST_CURSOR_MODE_EMBEDDED |
+                                          SCREEN_CAST_CURSOR_MODE_METADATA,
                 NULL);
 
   if (!g_dbus_interface_skeleton_export (impl,
